<?php

namespace EventPortal\Domain\User\Models;

use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Domain\User\Notifications\ResetPasswordNotification;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use Illuminate\Notifications\Notification;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;
use Spatie\Permission\Traits\HasRoles;

class User extends Authenticatable implements AuditableInterface
{
    use Notifiable, AddToFillable, AddToCasts, Auditable, HasRoles;

    const KEY = 'user';

    /** @var array */
    protected $fillable = [
        'first_name',
        'last_name',
        'email',
        'password',
        'password_created_at',
    ];

    /** @var array<string> */
    protected $dates = [
        'password_created_at',
    ];

    public function delegate(): HasOne
    {
        return $this->HasOne(Delegate::class);
    }

    public function speaker(): HasOne
    {
        return $this->HasOne(Speaker::class);
    }

    public function admin(): HasOne
    {
        return $this->hasOne(Admin::class);
    }

    public function teams(): MorphToMany
    {
        return $this->admin->teams();
    }

    public function getFullNameAttribute(): string
    {
        return trim(implode(' ', [$this->first_name, $this->last_name]));
    }

    public function isAdmin(): bool
    {
        return $this->admin !== null;
    }

    public function isDelegate(): bool
    {
        return $this->delegate !== null;
    }

    /**
     * These are the attributes used to build the email recipient array. It should be in the
     * format: [email => name]. If the PA email is set, it will be sent there instead.
     * (The original delegate will be CC'd so they still see the notification.)
     *
     * @see EventPortal\Domain\Util\Notification\SystemNotification
     */
    // phpcs:disable Generic.CodeAnalysis.UnusedFunctionParameter
    public function routeNotificationForMail(?Notification $notification): array
    {
        // phpcs:enable
        if ($this->isDelegate() && $this->delegate->hasPA()) {
            return [$this->delegate->pa_email => $this->full_name];
        }

        return [$this->email => $this->full_name];
    }

    /**
     * @param  string  $token
     * @phpcs:disable SlevomatCodingStandard.TypeHints.ParameterTypeHint.MissingNativeTypeHint
     */
    public function sendPasswordResetNotification($token): void
    {
        $this->notify(new ResetPasswordNotification($token));
    }
}
